@@ -10,16 +10,21 @@
    as published by the Free Software Foundation; either version
    2 of the License, or (at your option) any later version.
 */
-/* $Id: cipe.h,v 1.38.2.1 2002/05/30 11:49:17 olaf Exp $ */
+/* $Id: cipe.h,v 1.60 2004/08/04 13:51:14 olaf81825 Exp $ */
 
 #ifndef _CIPE_H_
 #define _CIPE_H_
 
-#include "crypto.h"
+/* XXX FIX ME */
+#define userKeySize     16
+#define MAXIVSIZE       16
+
 #ifdef __KERNEL__
 #include <linux/if.h>
+#include <linux/in.h>
 #else
 #include <net/if.h>
+#include <netinet/in.h>
 #endif
 
 /*** The kernel/user IOCTL interface ***/
@@ -30,20 +35,23 @@
    device name in ifr_name and a pointer to the actual control struct
    in ifr_data. */
 
+#define CIPHER_MAXNAMLEN 32
+
 #if 0
 /* Get interface parameters. Currently unused */
 #define SIOCGIFCIPPAR   (SIOCDEVPRIVATE+0)
 struct  siocgifcippar {
     unsigned long       magic;
     /* SOCKS5 relayer */
-    unsigned long       sockshost;
-    unsigned short      socksport;
+    struct sockaddr_in	socks;
     /* Timeouts (in seconds) */
     int                 tmo_keyxchg;
     int                 tmo_keylife;
     /* Flags */
     int                 flags;
     int		        cttl;
+    /* Encryption algorithm */
+    char                cname[CIPHER_MAXNAMLEN];
 };
 #endif
 
@@ -52,14 +60,15 @@ struct  siocgifcippar {
 struct  siocsifcippar {
     unsigned long       magic;
     /* SOCKS5 relayer */
-    unsigned long       sockshost;
-    unsigned short      socksport;
+    struct sockaddr_in	socks;
     /* Timeouts (in seconds) */
     int                 tmo_keyxchg;
     int                 tmo_keylife;
     /* Flags */
     int                 flags;
     int		        cttl;
+    /* Encryption algorithm */
+    char                cname[CIPHER_MAXNAMLEN];
 };
 
 /* Set a key. */
@@ -92,11 +101,16 @@ struct  siocsifcipall {
     char                name[IFNAMSIZ];
 };
 
-/* Flag values. */
+/*
+ * Flag values. Use only the upper 8 bits for external flags.
+ * See CIPF_MASK_EXT below.
+ */
 #define CIPF_MAY_CLEAR          0x0100
 #define CIPF_MAY_STKEY          0x0200
 #define CIPF_MAY_DYNIP          0x0400
 #define CIPF_DO_CSUM		0x0800
+#define CIPF_IGNORE_DF		0x1000
+#define CIPF_FORCE_MTU		0x2000
 
 /*** Key exchange related definitions ***/
 
@@ -144,6 +158,8 @@ struct  siocsifcipall {
 #include <linux/udp.h>
 #include <net/sock.h>
 #include <linux/version.h>
+#include "cryptoapi.h"
+
 #ifndef KERNEL_VERSION
 #define KERNEL_VERSION(a,b,c) (((a) << 16) + ((b) << 8) + (c))
 #endif
@@ -153,8 +169,15 @@ struct  siocsifcipall {
 #if LINUX_VERSION_CODE >= KERNEL_VERSION(2,3,0)
 #define LINUX_23
 #endif
-#if (__GNUC__ > 2 || (__GNUC__ == 2 && __GNUC_MINOR__ >= 91)) && defined(__i386__)
-#define REGPARM /* __attribute__((regparm,3)) XX needs testing */
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,4,0)
+#define LINUX_24
+#endif
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,5,0)
+#define LINUX_25
+#endif
+
+#if defined(USE_REGPARM) && (__GNUC__ > 2 || (__GNUC__ == 2 && __GNUC_MINOR__ >= 91)) && defined(__i386__)
+#define REGPARM __attribute__((regparm(3)))
 #else
 #define REGPARM
 #endif
@@ -165,7 +188,7 @@ struct  siocsifcipall {
 #endif
 #endif
 
-#if 1 /* Lock tracing */
+#if 0 /* Lock tracing */
 #define DOLOCK(s) ({ printk(KERN_DEBUG DEVNAME ": " #s " at %s:%d\n", \
 			    __FILE__, __LINE__); s; })
 #else
@@ -212,6 +235,10 @@ extern struct semaphore cipe_alloc_sem;
 #define DEV_STATS  enet_statistics
 #endif
 
+#ifndef LINUX_21
+typedef __u32 u_int32_t;
+#endif
+
 #if LINUX_VERSION_CODE < KERNEL_VERSION(2,1,127)
 #define timeout_t unsigned long
 #else
@@ -226,15 +253,63 @@ extern struct semaphore cipe_alloc_sem;
 #define get_fast_time do_gettimeofday
 #endif
 
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2,5,71)
+#define sk_shutdown shutdown
+#define sk_state state
+#define sk_zapped zapped
+#define sk_err err
+#define sk_error_report error_report
+#define sk_user_data user_data
+#define sk_prot prot
+#define sk_stamp stamp
+#define sk_no_check no_check
+#define sk_bound_dev_if bound_dev_if
+#endif
+
+#if LINUX_VERSION_CODE > KERNEL_VERSION(2,5,72)
+#define skb_linearize __skb_linearize
+#endif
+
+#ifdef LINUX_25
+#define CURRENT_TIME_SEC (xtime.tv_sec)
+#define ip_send          dst_output
+#else
+#define CURRENT_TIME_SEC (CURRENT_TIME)
+#define SOCKET_I(inode)  (&(inode)->u.socket_i)
+/* XX: the SCTP headers in 2.4 contain definitions for the following
+   two macros which differ from those needed in CIPE */
+#ifdef dst_pmtu
+#undef dst_pmtu
+#endif
+#define dst_pmtu(dst)    ((dst)->pmtu)
+#ifdef inet_sk
+#undef inet_sk
+#endif
+#define inet_sk(s)       (s)
+#endif
+
+#ifndef next_task
+#define next_task(p) ((p)->next_task)
+#endif
+
+#ifndef offsetof
+#define offsetof(TYPE, MEMBER) ((size_t) &((TYPE *)0)->MEMBER)
+#endif
+
+#ifndef BUG
+#define BUG() do { \
+        printk(KERN_CRIT "kernel BUG at %s:%d!\n", __FILE__, __LINE__); \
+        *((int*)0)=0xbeefdead; } while (0)
+#endif
+#ifndef BUG_ON
+#define BUG_ON(condition) do { if ((condition)!=0) BUG(); } while(0)
+#endif
+
 /* The header we add to each packet */
 #ifdef VER_ETH
-#define cipehdrlen (sizeof(struct iphdr)+sizeof(struct udphdr)+blockSize+ETH_HLEN)
+#define cipehdrlen (MAXIVSIZE+sizeof(struct iphdr)+sizeof(struct udphdr)+ETH_HLEN)
 #else
-#ifdef VER_SHORT
-#define cipehdrlen (sizeof(struct iphdr)+sizeof(struct udphdr))
-#else
-#define cipehdrlen (sizeof(struct iphdr)+sizeof(struct udphdr)+blockSize)
-#endif
+#define cipehdrlen (MAXIVSIZE+sizeof(struct iphdr)+sizeof(struct udphdr))
 #endif
 /* ...plus a real hardware header (common case) */
 #define cipexhdrl  (cipehdrlen+((ETH_HLEN+15)&~15))
@@ -259,25 +334,24 @@ struct cipe {
     __u32               sockshost;
     __u16               socksport;
     short	        cttl;
-#ifdef Crypto_IDEA
-    Key                 key_e, key_d, skey_e, rkey_d;
-#endif
-#ifdef Crypto_Blowfish
-    Key                 key, skey, rkey;
-    #define key_e       key
-    #define key_d       key
-    #define skey_e      skey
-    #define rkey_d      rkey
-#endif
     unsigned long       tmo_keyxchg;
     unsigned long       tmo_keylife;
+    struct crypto_tfm   *key, *skey, *rkey;
      /* Internal */
     unsigned long       timekx;
     unsigned long       timeskey;
     unsigned long       timerkey;
     int                 cntskey;
     int                 cntrkey;
+#ifdef LINUX_25
+    struct inet_sock    *sock;
+#define SOCK(c) ((struct sock*)((c)->sock))
+#define IOPT(c) (&(c)->sock->inet)
+#else
     struct sock         *sock;
+#define SOCK(c) ((c)->sock)
+#define IOPT(c) ((c)->sock)
+#endif
     int                 flags;
 #ifdef LINUX_21
     char                recursion;
@@ -294,10 +368,11 @@ struct cipe {
     struct proto        cipe_proto;
 };
 
-/* Flag values (internally used) */
+/* Flag values, internally used - only lower 8 bits */
 #define CIPF_HAVE_KEY           0x0001
 #define CIPF_HAVE_SKEY          0x0002
 #define CIPF_HAVE_RKEY          0x0004
+#define CIPF_NOTIFY_DERR        0x0080
 #define CIPF_MASK_INT           0x00FF
 #define CIPF_MASK_EXT           0xFF00
 
@@ -310,11 +385,11 @@ struct cipe {
 
 /* Master control struct */
 struct cipe_ctrl {
+    struct cipe         cipe; /* must be the first element */
+    struct NET_DEVICE 	dev;
 #ifndef HAVE_DEVNAME_ARRAY
     char                name[IFNAMSIZ];
 #endif
-    struct cipe         cipe;
-    struct NET_DEVICE 	dev;
 };
 
 extern struct cipe_ctrl **cipe_ctrls;
@@ -351,7 +426,7 @@ extern spinlock_t cipe_printk_lock;
 #define DEB_PKOU        64
 #define DEB_CHKP	128
 
-#define dprintk(l,p)	if(cipe_debug&l){LOCK_PRINTK; printk p; UNLOCK_PRINTK;}
+#define dprintk(l,p)	if(cipe_debug&(l)){LOCK_PRINTK; printk p; UNLOCK_PRINTK;}
 
 #else
 #define dprintk(l,p)	/* nop */
@@ -367,11 +442,26 @@ extern spinlock_t cipe_printk_lock;
 #define CHECKPOINT	/* nop */
 #endif
 
+static inline void nf_conntrack_null(struct sk_buff *skb)
+{
+#ifdef CONFIG_NETFILTER
+    nf_conntrack_put(skb->nfct);
+    skb->nfct = NULL;
+#ifdef CONFIG_NETFILTER_DEBUG
+    skb->nf_debug = 0;
+#endif
+#endif
+}
+
 /* internal routines */
 /* module.c */
-extern void cipe_use_module(void);
+extern int cipe_use_module(void);
 extern void cipe_unuse_module(void);
+#ifdef LINUX_25
+#define cipe_check_kernel() (0)
+#else
 extern int cipe_check_kernel(void);
+#endif
 /* device.c */
 extern void cipe_prnpad(unsigned char *buf, int len) REGPARM;
 extern void cipe_close(struct cipe *c);
@@ -382,6 +472,8 @@ extern int cipe_attach(struct NET_DEVICE *dev, struct siocsifcipatt *parm)
 extern void cipe_fakenkey(struct cipe *c, char typ) REGPARM;
 /* output.c */
 #ifdef DEBUG
+extern void cipe_hexdump(const unsigned char *bp, unsigned int len)
+    REGPARM;
 extern void cipe_dump_packet(char *title, struct sk_buff *skb, int dumpskb)
     REGPARM;
 #endif
@@ -391,25 +483,44 @@ extern void cipe_encrypt(struct cipe *c, unsigned char *buf,
 			 int *len, int typcode) REGPARM;
 extern unsigned short cipe_decrypt(struct cipe *c, unsigned char *buf,
 				   int *len) REGPARM;
-#ifndef VER_SHORT
-extern void cipe_cryptpad(unsigned char *buf) REGPARM;
-extern void cipe_cryptpad_iv(unsigned char *buf) REGPARM;
-#endif
+extern void cipe_cryptpad(unsigned char *buf, int len) REGPARM;
+extern void cipe_cryptpad_iv(unsigned char *buf, int len) REGPARM;
 
 #endif /* __KERNEL__ */
 
-#ifdef VER_CRC32
 /* crc32.c */
-extern unsigned long crc32(const unsigned char *s, unsigned int len);
+#if defined(__KERNEL__) && defined(HAVE_KERNEL_CRC)
+#include <linux/crc32.h>
 #else
-/* crc.c */
-extern unsigned short block_crc(unsigned char *d, int len);
+extern u_int32_t crc32(u_int32_t seed,
+                       const unsigned char *s, unsigned int len);
 #endif
 
 #define MIN(a,b) (((a)<(b))?(a):(b))
 
+#if     ProtocolVersion == 3
+#define VERNAME "c"
+
+#elif   ProtocolVersion == 4
+#define VERNAME "d"
+#define VER_ETH
+
+#else
+#error  "Must specify correct ProtocolVersion"
+#endif
+
+#ifdef  Crypto_Blowfish
+#define CRNAME			"b"
+#define CRNAMEC			'b'
+#define CRNUM			1
+#else
+#define CRNAME			""
+#define CRNAMEC			'@'
+#define CRNUM			2
+#endif
+
 #ifndef DEVNAME
 #define DEVNAME "cip" VERNAME CRNAME
 #endif
 
-#endif _CIPE_H_
+#endif /* _CIPE_H_ */
