@@ -10,7 +10,7 @@
    as published by the Free Software Foundation; either version
    2 of the License, or (at your option) any later version.
 */
-/* $Id: output.c,v 1.40.2.1 2001/05/01 21:23:42 olaf Exp $ */
+/* $Id: output.c,v 1.52 2004/03/06 22:16:41 olaf81825 Exp $ */
 
 #include "cipe.h"
 
@@ -20,6 +20,10 @@
 #include <linux/if_arp.h>
 #include <linux/socket.h>
 #include <linux/version.h>
+#ifdef LINUX_24
+#include <linux/netfilter.h>
+#include <linux/netfilter_ipv4.h>
+#endif
 
 #ifdef DEBUG
 
@@ -83,6 +87,14 @@ struct sk_buff *skb_copy_expand(struct sk_buff *skb, int max_headroom,
 }
 #endif
 
+#ifdef LINUX_24
+/* Need this wrapper because NF_HOOK takes the function address */
+static inline int do_ip_send(struct sk_buff *skb)
+{
+	return ip_send(skb);
+}
+#endif
+
 int cipe_xmit(struct sk_buff *skb, struct NET_DEVICE *dev)
 {
         struct cipe *tunnel = (struct cipe*)(dev->priv);
@@ -108,6 +120,11 @@ int cipe_xmit(struct sk_buff *skb, struct NET_DEVICE *dev)
                    ": cipe_xmit called with wrong struct\n");
             return 0;
         }
+	if (tunnel->recursion++) {
+        	printk(KERN_ERR "%s: cipe_xmit reentrance\n", dev->name);
+		tunnel->stat.collisions++;
+		goto tx_error_out;
+	}
 #ifdef DEBUG
         if (cipe_debug&DEB_PKOU)
             cipe_dump_packet("original", skb, 1);
@@ -133,17 +150,13 @@ int cipe_xmit(struct sk_buff *skb, struct NET_DEVICE *dev)
 		goto tx_error_out;
 	    }
 	    tos = old_iph->tos;
-	    df = old_iph->frag_off&__constant_htons(IP_DF);
+	    if (!tunnel->flags&CIPF_IGNORE_DF)
+	    	df = old_iph->frag_off&__constant_htons(IP_DF);
 	    ttl = tunnel->cttl ? tunnel->cttl : old_iph->ttl;
 	} else {
 	    ttl = tunnel->cttl ? tunnel->cttl : 64; /* XX */
 	}
 
-	if (tunnel->recursion++) {
-        	printk(KERN_ERR "%s: cipe_xmit reentrance\n", dev->name);
-		tunnel->stat.collisions++;
-		goto tx_error_out;
-	}
 #ifndef VER_ETH
 	if (skb->protocol != __constant_htons(ETH_P_IP))
 		goto tx_error_out;
@@ -154,6 +167,22 @@ int cipe_xmit(struct sk_buff *skb, struct NET_DEVICE *dev)
                           cipe_ntoa(0, dst), cipe_ntoa(1, tunnel->myaddr),
                           RT_TOS(tos), tunnel->sock->bound_dev_if));
 #endif
+#ifdef LINUX_25
+	{
+            struct flowi fl = { .oif = SOCK(tunnel)->sk_bound_dev_if,
+                                .nl_u = { .ip4_u =
+                                          { .daddr = dst,
+                                            .saddr = IOPT(tunnel)->rcv_saddr,
+                                            .tos = RT_TOS(tos) } },
+                                .proto = IPPROTO_UDP };
+            if (ip_route_output_key(&rt, &fl)) {
+                dprintk(DEB_OUT, (KERN_NOTICE "%s: no route\n", dev->name));
+                tunnel->stat.tx_carrier_errors++;
+                dst_link_failure(skb);
+                goto tx_error_out;
+            }
+	}
+#else
 	if (ip_route_output(&rt, dst, tunnel->sock->rcv_saddr, RT_TOS(tos),
                             tunnel->sock->bound_dev_if)) {
             dprintk(DEB_OUT, (KERN_NOTICE "%s: no route\n", dev->name));
@@ -161,10 +190,11 @@ int cipe_xmit(struct sk_buff *skb, struct NET_DEVICE *dev)
             dst_link_failure(skb);
             goto tx_error_out;
         }
+#endif
         if (rt->rt_src!=tunnel->myaddr) {
             printk(KERN_NOTICE "%s: changing my address: %s\n", dev->name,
                    cipe_ntoa(rt->rt_src));
-            tunnel->myaddr=tunnel->sock->saddr=rt->rt_src;
+            tunnel->myaddr=IOPT(tunnel)->saddr=rt->rt_src;
 	}
 
 	tdev = rt->u.dst.dev;
@@ -177,18 +207,28 @@ int cipe_xmit(struct sk_buff *skb, struct NET_DEVICE *dev)
 		goto tx_error;
 	}
 
-        mtu = rt->u.dst.pmtu - (cipehdrlen+cipefootlen);
-        if (tunnel->sockshost)
-            mtu -= sizeof(struct sockshdr);
+	if (tunnel->flags&CIPF_FORCE_MTU) {
+		mtu = dev->mtu;
+	} else {
+        	mtu = dst_pmtu(&rt->u.dst) - (cipehdrlen+cipefootlen);
+        	if (tunnel->sockshost)
+            		mtu -= sizeof(struct sockshdr);
+        }
 
-        dprintk(DEB_OUT, (KERN_DEBUG "pmtu=%d dmtu=%d size=%d\n",
-                          mtu, tdev->mtu, skb->len));
+        dprintk(DEB_OUT, (KERN_DEBUG "pmtu=%d dmtu=%d size=%d df=%d forcemtu=%d ignoredf=%d\n",
+                          mtu, tdev->mtu, skb->len, df,
+                          (tunnel->flags&CIPF_FORCE_MTU)?1:0,
+                          (tunnel->flags&CIPF_IGNORE_DF)?1:0));
 
 	if (mtu < 68) {
         	printk(KERN_ERR "%s: MTU too small\n", dev->name);
 		tunnel->stat.collisions++;
 		goto tx_error;
 	}
+#ifdef LINUX_25
+	if (skb->dst)
+		skb->dst->ops->update_pmtu(skb->dst, mtu);
+#else
 	if (skb->dst && mtu < skb->dst->pmtu) {
 		skb->dst->pmtu = mtu;
         	dprintk(DEB_OUT, (KERN_NOTICE "%s: adjusting PMTU\n", dev->name));
@@ -198,8 +238,9 @@ int cipe_xmit(struct sk_buff *skb, struct NET_DEVICE *dev)
                 goto tx_error;
 #endif
         }
+#endif
 
-	if (old_iph && (old_iph->frag_off&__constant_htons(IP_DF)) && mtu < ntohs(old_iph->tot_len)) {
+	if (old_iph && df && mtu < ntohs(old_iph->tot_len)) {
         	dprintk(DEB_OUT, (KERN_NOTICE "%s: fragmentation needed\n", dev->name));
 		icmp_send(skb, ICMP_DEST_UNREACH, ICMP_FRAG_NEEDED, htonl(mtu));
 		goto tx_error;
@@ -228,11 +269,10 @@ int cipe_xmit(struct sk_buff *skb, struct NET_DEVICE *dev)
 	skb->protocol = htons(ETH_P_IP);
 
 	if (tunnel->flags&CIPF_HAVE_KEY) {
-#ifndef VER_SHORT
 	    /* Add an IV */
-	    cipe_cryptpad_iv(skb_push(skb, blockSize));
-	    length+=blockSize;
-#endif
+            int ivs = crypto_tfm_alg_ivsize(tunnel->key);
+	    cipe_cryptpad_iv(skb_push(skb, ivs), ivs);
+	    length+=ivs;
             if (!tunnel->flags&CIPF_MAY_STKEY && !tunnel->flags&CIPF_HAVE_SKEY)
                 /* Attempt to encrypt data using invalid static key */
                 goto tx_error;
@@ -277,13 +317,13 @@ int cipe_xmit(struct sk_buff *skb, struct NET_DEVICE *dev)
 	iph->ihl        =	sizeof(struct iphdr)>>2;
 	iph->tos        =	tos;
 	iph->tot_len    =	htons(skb->len);
-	ip_ident(iph, &rt->u.dst);
         iph->frag_off   =	df;
         iph->ttl        =	ttl;
 	iph->protocol   =	IPPROTO_UDP;
 	iph->saddr      =	rt->rt_src;
 	iph->daddr      =	rt->rt_dst;
 
+	ip_ident(iph, &rt->u.dst);
 	ip_send_check(iph);
 
         udph->source = tunnel->myport;
@@ -301,8 +341,6 @@ int cipe_xmit(struct sk_buff *skb, struct NET_DEVICE *dev)
 		udph->check=-1;
 	}
 
-	tunnel->stat.tx_bytes += skb->len;
-	tunnel->stat.tx_packets++;
         dprintk(DEB_OUT, (KERN_INFO
                           "%s: sending %d from %s:%d to %s:%d\n",
                           dev->name, skb->len,
@@ -322,7 +360,25 @@ int cipe_xmit(struct sk_buff *skb, struct NET_DEVICE *dev)
         if (cipe_debug&DEB_PKOU)
             cipe_dump_packet("sending", skb, 0);
 #endif
-	ip_send(skb);
+        nf_conntrack_null(skb);
+        skb->ip_summed = CHECKSUM_NONE;
+#ifdef LINUX_24
+        {
+            int err = NF_HOOK(PF_INET, NF_IP_LOCAL_OUT, skb, NULL,
+                              rt->u.dst.dev, do_ip_send);
+            if (err==NET_XMIT_SUCCESS || err==NET_XMIT_CN) {
+                tunnel->stat.tx_bytes += skb->len;
+                tunnel->stat.tx_packets++;
+            } else {
+		tunnel->stat.tx_errors++;
+		tunnel->stat.tx_aborted_errors++;
+            }
+        }
+#else
+        ip_send(skb);
+        tunnel->stat.tx_bytes += skb->len;
+        tunnel->stat.tx_packets++;
+#endif
         tunnel->recursion--;
 	return 0;
 
@@ -476,11 +532,10 @@ int cipe_xmit(struct sk_buff *skb, struct NET_DEVICE *dev)
 #endif
         length = skb->len;
 	if (c->flags&CIPF_HAVE_KEY) {
-#ifndef VER_SHORT
 	    /* Add an IV */
-	    cipe_cryptpad(skb_push(skb, blockSize));
-	    length+=blockSize;
-#endif
+	    cipe_cryptpad(skb_push(skb, c->cipher->ivsize),
+                          c->cipher->ivsize);
+	    length+=c->cipher->ivsize;
             if (!c->flags&CIPF_MAY_STKEY && !c->flags&CIPF_HAVE_SKEY)
                 /* Attempt to encrypt data using invalid static key */
                 goto error;
