@@ -10,7 +10,7 @@
    as published by the Free Software Foundation; either version
    2 of the License, or (at your option) any later version.
 */
-/* $Id: sock.c,v 1.33.2.2 2002/05/30 11:49:17 olaf Exp $ */
+/* $Id: sock.c,v 1.47 2004/03/06 22:16:41 olaf81825 Exp $ */
 
 #include "cipe.h"
 
@@ -18,6 +18,7 @@
 #include <linux/sched.h>
 #include <net/sock.h>
 #include <net/ip.h>
+#include <linux/tcp.h>
 #include <linux/etherdevice.h>
 #ifdef LINUX_21
 #include <asm/uaccess.h>
@@ -43,23 +44,21 @@ typedef unsigned short mm_segment_t;
 #endif
 
 /* init struct cipe *c based on struct sock *sk */
-#define SOCKTOC(nam,sk,c,r) \
-    struct cipe *c=(struct cipe *)sk->user_data; \
-    if ((!c) || (c->magic!=CIPE_MAGIC)) { \
-	 printk(KERN_ERR "Ouch: SOCKTOC " nam "\n"); return r; } \
-    else { \
-         dprintk(DEB_CALL, (KERN_INFO "%s: " nam "\n", c->dev->name)); }
+#define SOCKTOC(nam,sk,c) \
+    struct cipe *c=(struct cipe *)sk->sk_user_data; \
+    BUG_ON((!c) || (c->magic!=CIPE_MAGIC)); \
+    dprintk(DEB_CALL, (KERN_INFO "%s: " nam "\n", c->dev->name));
 
 
 /* Close the socket */
 void cipe_sock_close(struct sock *sock, timeout_t timeout)
 {
-    SOCKTOC("sock_close",sock,c,);
+    SOCKTOC("sock_close",sock,c);
     c->sock=NULL;
 
     /* Put back the old protocol block and let it do the close */
-    sock->prot=c->udp_prot;
-    sock->prot->close(sock, timeout);
+    sock->sk_prot=c->udp_prot;
+    sock->sk_prot->close(sock, timeout);
     if (c->dev->flags&IFF_UP) {
 	rtnl_LOCK();
 	dev_close(c->dev);
@@ -72,9 +71,18 @@ void cipe_sock_close(struct sock *sock, timeout_t timeout)
 /* Anything we send to this socket is in fact a key exchange block.
    Encode and encrypt it accordingly.
 */
-int cipe_sendmsg(struct sock *sock, struct msghdr *msg, int len
+int cipe_sendmsg(
+#ifdef LINUX_25
+    struct kiocb *iocb,
+#endif
+    struct sock *sock, struct msghdr *msg
+#ifdef LINUX_25
+    , size_t len
+#else
+    , int len
+#endif
 #ifndef LINUX_21
-                 , int nonblock, int flags
+    , int nonblock, int flags
 #endif
     )
 {
@@ -82,43 +90,32 @@ int cipe_sendmsg(struct sock *sock, struct msghdr *msg, int len
     struct iovec myio[2];
     struct sockaddr_in sa;
     struct sockshdr sh;
-    int e, n=0;
-    unsigned char buf[KEYXCHGBLKMAX+blockSize];
-    SOCKTOC("cipe_sendmsg",sock,c,-ENOSYS);
+    int e, n=0, ivs;
+    unsigned char buf[KEYXCHGBLKMAX];
+    SOCKTOC("cipe_sendmsg",sock,c);
+    ivs=crypto_tfm_alg_ivsize(c->key);
 
-    if (len>KEYXCHGBLKMAX)
+    if (!(c->flags&CIPF_HAVE_KEY) && !(msg->msg_flags&MSG_OOB))
+        return -EINVAL;
+    if (len>KEYXCHGBLKMAX-ivs)
 	return -EMSGSIZE;
-#ifdef VER_SHORT
-    if (msg->msg_flags&MSG_OOB)
-	return -EINVAL;
-#endif
     cipe_prnpad(buf, sizeof(buf));
-#ifdef VER_SHORT
-    memcpy_fromiovec(buf, msg->msg_iov, len);
-#else
-    memcpy_fromiovec(buf+blockSize, msg->msg_iov, len);
-    len+=blockSize;
-#endif
+    memcpy_fromiovec(buf+ivs, msg->msg_iov, len);
+    len+=ivs;
     if (!(msg->msg_flags&MSG_OOB)) {
-	if (buf[0
-#ifndef VER_SHORT
-	       +blockSize
-#endif
-	                 ]>=CT_DUMMY) {
-	    if (!c->flags&CIPF_HAVE_KEY)
-		return -EINVAL;
+	if (buf[ivs]>=CT_DUMMY) {
 	    buf[KEYXCHGTSPOS-1]='\0';
 	} else {
 	    if (len<KEYXCHGBLKMIN)
 		return -EINVAL;
 	}
     }
-    (*(__u32 *)(buf+KEYXCHGTSPOS))=htonl(CURRENT_TIME); /* timestamp */
+    (*(__u32 *)(buf+KEYXCHGTSPOS))=htonl(CURRENT_TIME_SEC); /* timestamp */
     if (msg->msg_flags&MSG_OOB) {
-	memset(buf, 0, blockSize);
+	memset(buf, 0, ivs);
     } else {
 	len=KEYXCHGBLKMIN+buf[sizeof(buf)-1]; /* random */
-	cipe_encrypt(c, buf, &len, TW_NEWKEY);
+	cipe_encrypt(c, buf, (int*)&len, TW_NEWKEY);
     }
 
     sa.sin_family=AF_INET;
@@ -137,6 +134,9 @@ int cipe_sendmsg(struct sock *sock, struct msghdr *msg, int len
     }
     myio[n].iov_base=&buf;
     myio[n].iov_len=len;
+    if (c->sockshost)
+        len+=sizeof(struct sockshdr);
+
     /* mymsg=*msg; */
     mymsg.msg_name=&sa;
     mymsg.msg_namelen=sizeof(sa);
@@ -151,16 +151,18 @@ int cipe_sendmsg(struct sock *sock, struct msghdr *msg, int len
     {
         mm_segment_t fs=get_fs();
         set_fs(get_ds());
-        if (c->sockshost)
-            len+=sizeof(struct sockshdr);
-        dprintk(DEB_KXC, (KERN_INFO "%s: real sendmsg len %d text=%04x...\n",
-                          c->dev->name, len,
+        dprintk(DEB_KXC, (KERN_INFO "%s: real sendmsg len %ld text=%04x...\n",
+                          c->dev->name, (long)len,
                           ntohs(*((unsigned short *)(myio[n].iov_base)))));
-        e=c->udp_prot->sendmsg(sock, &mymsg, len
-#ifndef LINUX_21
-                               , nonblock, flags
+#ifdef LINUX_25
+        e=c->udp_prot->sendmsg(NULL, sock, &mymsg, len);
+#else
+#ifdef LINUX_21
+        e=c->udp_prot->sendmsg(sock, &mymsg, len);
+#else
+        e=c->udp_prot->sendmsg(sock, &mymsg, len, nonblock, flags);
+#endif
 #endif
-            );
         set_fs(fs);
     }
     return e;
@@ -211,7 +213,8 @@ struct sk_buff *cipe_decrypt_skb(struct cipe *c, struct sock *sk,
 	goto framerr;
     }
 #endif
-    if (length<cipehdrlen+(c->sockshost?sizeof(struct sockshdr):0)) {
+    if (length<9+(c->sockshost?sizeof(struct sockshdr):0)) {
+        /* XX hardcoded IV size */
         printk(KERN_INFO "%s: got short packet from %s\n", c->dev->name,
                cipe_ntoa(saddr(skb)));
 	goto framerr;
@@ -250,6 +253,20 @@ struct sk_buff *cipe_decrypt_skb(struct cipe *c, struct sock *sk,
 #endif
     rsport=skb->h.uh->source;
     skb_put(n,skb->len);
+#ifdef LINUX_24
+    if (skb_is_nonlinear(skb)) {
+#ifdef DEBUG
+       dprintk(DEB_INP, (KERN_INFO DEVNAME
+                         ": skb is non linear, will linearize\n"));
+#endif
+       if (skb_linearize(skb, GFP_KERNEL) != 0)
+       {
+               printk(KERN_WARNING "%s: cipe_decrypt_skb: out of memory\n",
+                      c->dev->name);
+               goto error;
+       }
+    }
+#endif
     memcpy(n->data, skb->h.raw, skb->len);
     n->h.uh=(struct udphdr *)n->data;
 
@@ -283,11 +300,11 @@ struct sk_buff *cipe_decrypt_skb(struct cipe *c, struct sock *sk,
 	n->h.uh->source=rsport;
     }
 
-#ifndef VER_SHORT
     if (!(*((__u32 *)n->data) & htonl(0x7FFFFFFF))) {
-	dprintk(DEB_INP, (KERN_INFO "got unencrypted packet (iv=0) len=%d\n",
-			  length));
-	skb_pull(n, blockSize);
+	dprintk(DEB_INP,
+                (KERN_INFO "got unencrypted control packet (iv=0) len=%d\n",
+                 length));
+	skb_pull(n, 8); /* XX hardcoded IV size */
 	/* XX not sure about TW_CTRL handling... */
         switch (*n->data) {
             /* Restrict what can be received unencrypted. */
@@ -299,6 +316,7 @@ struct sk_buff *cipe_decrypt_skb(struct cipe *c, struct sock *sk,
                    c->dev->name, *n->data);
             goto error;
         }
+
 #ifdef LINUX_21
 	get_fast_time(&n->stamp);
 #endif
@@ -306,9 +324,8 @@ struct sk_buff *cipe_decrypt_skb(struct cipe *c, struct sock *sk,
 	n->h.uh->check=0;
 	return n;
     }
-#endif
     if (c->flags&CIPF_HAVE_KEY) {
-	if (length%blockSize) {
+	if (length%crypto_tfm_alg_ivsize(c->key)) {
 	    printk(KERN_INFO "%s: got bogus length=%d\n", c->dev->name,
 		   length);
 	    goto error;
@@ -384,9 +401,13 @@ struct sk_buff *cipe_decrypt_skb(struct cipe *c, struct sock *sk,
 		      atomic_read(&n->dev->refcnt)));
 #endif
 #endif
-
+    nf_conntrack_null(skb);
     /* requeue */
     netif_rx(n);
+#ifdef LINUX_24
+    /* raise a softirq to process the new packet asap */
+    do_softirq();
+#endif
 #ifdef LINUX_21
     c->stat.rx_bytes+=skb->len; /* count carrier-level bytes */
 #endif
@@ -408,14 +429,23 @@ struct sk_buff *cipe_decrypt_skb(struct cipe *c, struct sock *sk,
    UDP header.
 */
 
-int cipe_recvmsg(struct sock *sk, struct msghdr *msg, int len,
-		 int noblock, int flags,int *addr_len)
+int cipe_recvmsg(
+#ifdef LINUX_25
+    struct kiocb *iocb,
+#endif
+    struct sock *sk, struct msghdr *msg
+#ifdef LINUX_25
+    , size_t len
+#else
+    , int len
+#endif
+    , int noblock, int flags,int *addr_len)
 {
   	int copied;
   	struct sk_buff *skb, *skn;
   	int er=0;
   	struct sockaddr_in *sin=(struct sockaddr_in *)msg->msg_name;
-	SOCKTOC("cipe_recvmsg",sk,c,-ENOSYS);
+	SOCKTOC("cipe_recvmsg",sk,c);
 
 	/*
 	 *	Check any passed addresses
@@ -473,22 +503,13 @@ int cipe_recvmsg(struct sock *sk, struct msghdr *msg, int len,
   	 *	FIXME : should use udp header size info value
   	 */
 #ifdef LINUX_21
-#if 0
-	er = skb_copy_datagram_iovec(skb, skb->data-skb-h.raw,
-				     msg->msg_iov, copied);
-#else
 	er = memcpy_toiovec(msg->msg_iov, skb->data, copied);
-#endif
 	if (er<0)
 		goto out_free;
-#else
-#if 0
-        skb_copy_datagram_iovec(skb,of,msg->msg_iov,copied);
 #else
 	memcpy_toiovec(msg->msg_iov, skb->data, copied);
 #endif
-#endif
-	sk->stamp=skb->stamp;
+	sk->sk_stamp=skb->stamp;
 
 	/* Copy the address. */
 	if (sin
@@ -533,7 +554,7 @@ int cipe_attach(struct NET_DEVICE *dev, struct siocsifcipatt *parm)
     if (!(file=fget(parm->fd)))
         return(-EBADF);
     inode = file->f_dentry->d_inode;
-    if (!inode || !inode->i_sock || !(sock=&inode->u.socket_i)) {
+    if (!inode || !inode->i_sock || !(sock=SOCKET_I(inode))) {
         fput(file);
         return(-ENOTSOCK);
     }
@@ -549,37 +570,43 @@ int cipe_attach(struct NET_DEVICE *dev, struct siocsifcipatt *parm)
 	return(-EAFNOSUPPORT);
 
     sk=sock->sk;
-    if (sk->state!=TCP_ESTABLISHED)
+    if (sk->sk_state!=TCP_ESTABLISHED)
 	return(-ENOTCONN);
-    if (((c0=(struct cipe *)sk->user_data)) &&
+    if (((c0=(struct cipe *)sk->sk_user_data)) &&
 	(c0->magic==CIPE_MAGIC))
 	return(-EBUSY); /* socket is already attached */
 
-    cipe_use_module();
+    if (!cipe_use_module())
+        return(-ENODEV);
+
     c->owner=current->pid;
+#ifdef LINUX_25
+    c->sock=(struct inet_sock*)sk;
+#else
     c->sock=sk;
-    c->peeraddr=sk->daddr;
-    c->peerport=sk->dport;
-    c->myaddr=sk->saddr;
+#endif
+    c->peeraddr=inet_sk(sk)->daddr;
+    c->peerport=inet_sk(sk)->dport;
+    c->myaddr=inet_sk(sk)->saddr;
     if (c->flags&CIPF_MAY_DYNIP)
-        sk->rcv_saddr=0;
-    c->myport=htons(sk->num);
+        inet_sk(sk)->rcv_saddr=0;
+    c->myport=htons(inet_sk(sk)->num);
     /* Disconnect socket, we might receive from somewhere else */
-    sk->daddr=0;
-    sk->dport=0;
+    inet_sk(sk)->daddr=0;
+    inet_sk(sk)->dport=0;
 
     /* Fill an otherwise unused field in the sock struct with this info.
        This field is conveniently named and the kernel uses it only for RPC. */
-    sk->user_data=c;
-    sk->no_check=(c->flags&CIPF_DO_CSUM) ? 0 : 1;
+    sk->sk_user_data=c;
+    sk->sk_no_check=(c->flags&CIPF_DO_CSUM) ? 0 : 1;
 
     /* Set up new socket operations */
-    c->udp_prot=sk->prot;
-    c->cipe_proto=*sk->prot;
+    c->udp_prot=sk->sk_prot;
+    c->cipe_proto=*sk->sk_prot;
     c->cipe_proto.close=cipe_sock_close;
     c->cipe_proto.sendmsg=cipe_sendmsg;
     c->cipe_proto.recvmsg=cipe_recvmsg;
-    sk->prot=&c->cipe_proto;
+    sk->sk_prot=&c->cipe_proto;
 
 #if 0
     /* (Try to) Set our dummy hardware address from the IP address. */
@@ -620,6 +647,8 @@ int cipe_attach(struct NET_DEVICE *dev, struct siocsifcipatt *parm)
     cli();
     if (c->sock)
 	sreturn(-EBUSY);
+    if (!c->cipher)
+        sreturn(-ENXIO);
 
     /* Find the socket (from net/socket.c) */
     if (fd < 0 || fd >= NR_OPEN || !(file = current->files->fd[fd]))
@@ -639,7 +668,7 @@ int cipe_attach(struct NET_DEVICE *dev, struct siocsifcipatt *parm)
 	(c0->magic==CIPE_MAGIC))
 	sreturn(-EBUSY); /* socket is already attached */
 
-    cipe_use_module();
+    cipe_use_module(); /* cannot fail under 2.0 */
     c->owner=current->pid;
     c->sock=sk;
     c->peeraddr=sk->daddr;
@@ -715,9 +744,9 @@ void cipe_fakenkey(struct cipe *c, char typ)
        one _bit_... */
     skb->data[sizeof(struct udphdr)]=typ;
     (*(__u32 *)(skb->data+sizeof(struct udphdr)+KEYXCHGTSPOS))=
-        htonl(CURRENT_TIME); /* even need timestamp */
+        htonl(CURRENT_TIME_SEC); /* even need timestamp */
 
-    if (sock_queue_rcv_skb(c->sock, skb)<0) {
+    if (sock_queue_rcv_skb(SOCK(c), skb)<0) {
         printk(KERN_WARNING "%s: cipe_fakenkey: enqueuing failed\n",
                c->dev->name);
         kfreeskb(skb, FREE_WRITE);
